#!/usr/bin/env python3
# ----------------------------------------------------------------------------
#
# Copyright 2018 EMVA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# ----------------------------------------------------------------------------


# Standard library imports

# Related third party imports
from genicam2.gentl import InvalidParameterException
import numpy as np

# Local application/library specific imports


class Image:
    def __init__(self, parent=None, ndarray: np.ndarray=None):
        #
        super().__init__()

        #
        self._parent = parent
        self._ndarray = ndarray

    @property
    def width(self) -> int:
        try:
            width = self._parent.gentl_buffer.width
        except InvalidParameterException:
            width = self._parent.node_map.Width.value

        return width

    @property
    def height(self) -> int:
        try:
            height = self._parent.gentl_buffer.height
        except InvalidParameterException:
            height = self._parent.node_map.Height.value

        return height

    @property
    def pixel_format(self) -> str:
        try:
            pixel_format_int = self._parent.gentl_buffer.pixel_format
        except InvalidParameterException:
            return self._parent.node_map.PixelFormat.value
        else:
            return self._parent.node_map.PixelFormat.get_entry(
                int(pixel_format_int)
            ).symbolic

    @property
    def ndarray(self) -> np.ndarray:
        return self._ndarray




#!/usr/bin/env python3
# ----------------------------------------------------------------------------
#
# Copyright 2018 EMVA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# ----------------------------------------------------------------------------


# Standard library imports
import unittest

# Related third party imports

# Local application/library specific imports
from harvester.test.base_harvester import TestHarvesterCoreBase


class TestHarvesterCore(TestHarvesterCoreBase):
    @staticmethod
    def print_buffer(buffer):
        print('W: {0} x H: {1}, {2}\n{3}'.format(
            buffer.image.width,
            buffer.image.height,
            buffer.image.pixel_format,
            buffer.image.ndarray
        ))

    def test_harvester_core(self):
        #
        for i in range(10):
            print('---> {0}: started'.format(i))
            self._harvester.start_image_acquisition()
            j = 0
            # Run it as fast as possible.
            frames = 10
            while j < frames:
                if j % 2 == 0:
                    # Option 1: This way is secure and preferred.
                    try:
                        # We know we've started image acquisition but this
                        # try-except block is demonstrating a case where
                        # a client called fetch_buffer method even though
                        # he'd forgotten to start image acquisition.
                        with self._harvester.fetch_buffer() as buffer:
                            self.print_buffer(buffer)
                    except AttributeError:
                        # Harvester Core has not started image acquisition so
                        # calling fetch_buffer() raises AttributeError because
                        # None object is used for the with statement.
                        pass
                else:
                    # Option 2: You can manually do the same job but not
                    # recommended because you might forget to queue the
                    # buffer.
                    buffer = self._harvester.fetch_buffer()
                    self.print_buffer(buffer)
                    self._harvester.queue_buffer(buffer)

                #
                j += 1
            self._harvester.stop_image_acquisition()
            print('    <--- stopped {0}'.format(i))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# ----------------------------------------------------------------------------
#
# Copyright 2018 EMVA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# ----------------------------------------------------------------------------


# Standard library imports

# Related third party imports

# Local application/library specific imports


class ThreadBase:
    def __init__(self, mutex=None):
        #
        super().__init__()

        #
        self._is_running = False
        self._mutex = mutex

    def start(self):
        self._is_running = True
        self._start()

    def _start(self):
        raise NotImplementedError

    def stop(self):
        raise NotImplementedError

    def acquire(self):
        raise NotImplementedError

    def release(self):
        raise NotImplementedError

    @property
    def is_running(self):
        return self._is_running

    @is_running.setter
    def is_running(self, value):
        self._is_running = value

    @property
    def worker(self):
        raise NotImplementedError

    @worker.setter
    def worker(self, obj):
        raise NotImplementedError

    @property
    def mutex(self):
        raise NotImplementedError


class MutexLocker:
    def __init__(self, thread: ThreadBase=None):
        #
        super().__init__()

        #
        self._thread = thread
        self._locked_mutex = None

    def __enter__(self):
        #
        if self._thread is None:
            return None

        #
        self._locked_mutex = self._thread.acquire()
        return self._locked_mutex

    def __exit__(self, exc_type, exc_val, exc_tb):
        #
        if self._thread is None:
            return

        #
        self._thread.release()





#!/usr/bin/env python3
# ----------------------------------------------------------------------------
#
# Copyright 2018 EMVA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# ----------------------------------------------------------------------------


# Standard library imports
import sys
import unittest

# Related third party imports

# Local application/library specific imports
from harvester.core import Harvester
from harvester._private.core.helper.system import is_running_on_windows


class TestHarvesterCoreBase(unittest.TestCase):
    _path = 'C:/Users/z1533tel/dev/genicam/bin/Win64_x64/' \
        if is_running_on_windows() else \
        '/Users/kznr/dev/genicam/bin/Maci64_x64/'

    _filename = 'TLSimu.cti'

    sys.path.append(_path)

    def __init__(self, *args, **kwargs):
        #
        super().__init__(*args, **kwargs)

    def setUp(self):
        #
        super().setUp()

        #
        self._harvester = Harvester()
        self._harvester.add_cti_file(self._path + self._filename)
        self._harvester.update_device_info_list()
        self._harvester.connect_device(0)

    def tearDown(self):
        #
        self._harvester.reset()

        #
        super().tearDown()


if __name__ == '__main__':
    unittest.main()
